'''
 * Copyright: STEM Loyola
 * Date     : August 2019
 * ID       : 19.08-B2
 * Level    : 2 (Intermediate)
 *
 * Task     : In 2014, NECTA released the form four (CSEE) results in GPA and
 *            class format (Figure 2). This came after they had introduced "B+"
 *            and "E" grades during the 2013 results. Your task is to convert
 *            the results to the points and divisions format (Figure 3) that we
 *            are all familiar with and count students in each division. A text
 *            file "2014-CSEE-S0800.txt" contains the results in GPA format as
 *            shown in Figure 1. No one got division zero but we have added one
 *            division-zero results to ensure you write a complete program.
 *
 *      Note: (1) Points and divisions are calculated from the 7 best grades as
 *                shown in Figure 4 and Figure 5.
 *            (2) In order to get Division IV, a student must have at least
 *                2 Ds or 1 C. This means, for example, that 6 Es and 1 D
 *                (i.e. 41 pts) will still be Division 0.
 *
 * Solved By: STEM Loyola Team
 *     Email: stemloyola@gmail.com
 *     Form : N/A
 *    Stream: N/A
'''

FILE_NAME = "2014-CSEE-S0800.txt"

# Open the results file
try:
    resultsFile = open (FILE_NAME, "r")

except:
    # Terminate the program when the file could not be opened
    print ("Error: could not open the file \"{}\"".format(FILE_NAME))
    print ("Quitting...")

    exit(1)
    
totalDivisions = [0, 0, 0, 0, 0]  # Tracks students in each division

gradeToPoints = {"A":1, "B+":2, "B":3, "C":4, "D":5, "E":6, "F":7}  # Grade to points mapping

# Parse the results files and extract students data
for line in resultsFile:
    # Extract the student ID and gender
    line = line.strip()
    items = line.split(' ', 2)
    
    studID = items[0]
    gender = items[1]
    
    # Extract all available subjects
    results = line[13:].split("'")
    grades = results[1::2]
    
    # Compute and sum up the student's best seven points
    pointsList = []
    for grade in grades:
        pointsList.append(gradeToPoints[grade])
        
    pointsList.sort()
    
    points = sum(pointsList[:7])
    
    # Compute the student's division
    division = ""
    
    if points <= 17:
        division = "I"
        totalDivisions[1] += 1
    elif points <= 24:
        division = "II"
        totalDivisions[2] += 1
    elif points <= 31:
        division = "III"
        totalDivisions[3] += 1
    elif points <= 46:
        # At least one C or two D's are needed to get division 4 even
        # if a student scores the right points
        if pointsList[0] <= 4 or pointsList[0] + pointsList[1] <= 10:
            division = "IV"
            totalDivisions[4] += 1
        else:
            division = "0"
            totalDivisions[0] += 1
    else:
        division = "0"
        totalDivisions[0] += 1
    
    # Display the student's results
    print(studID, gender, points, division, sep="\t")
    
# Display the overall results summary
print("\nResults summary:", end="\n\t")
print("Division I:     ", totalDivisions[1], end="\n\t")
print("Division II:    ", totalDivisions[2], end="\n\t")
print("Division III:   ", totalDivisions[3], end="\n\t")
print("Division IV:    ", totalDivisions[4], end="\n\t")
print("Division 0 (*): ", totalDivisions[0], end="\n\t") 

# Close open resources
resultsFile.close()
